#include <unistd.h>
#include <time.h>

#include "simple.hxx"

Ref<MainObj> Main ;


const char* DB_NAME = "test" ;

TstObj::TstObj(int offs, int first) {

  char string[512] ;
  
  this->_offs = offs ;
  this->_first = first ;
  sprintf(string, "Object number %d", offs + first) ;
  this->_string = string ;

  if ( (random() % 100) == 0) {
    this->_largeSize = random() % DB_MAX_LARGE1 ;
  } else {
    this->_largeSize = random() % DB_MAX_LARGE2 ;
  }
  this->_large.alloc(this->_largeSize) ;
  this->_large[this->_largeSize - 1] = (char)(offs % 256) ;

  yoo_setname(this, this->_string) ;
  
  for (int i = 0; i < offs; i++) {
    this->_list += i ;
  }
}


TstObj::TstObj(const TstObj& obj) {
  *this = obj ;
}


TstObj& TstObj::operator= (const TstObj& obj) {
  
  this->_offs = obj._offs ;
  this->_first = obj._first ;
  this->_string = obj._string ;
  this->_largeSize = obj._largeSize ;
  this->_large.alloc(obj._largeSize) ;
  this->_large[this->_largeSize - 1] = this->_offs % 256 ;
  this->_list = obj._list ;
}


int TstObj::modify(const Ref<TstObj>& obj) {

  TstObj tmp = *this ;
  *this = *obj ;
  *obj = tmp ;
  return 1 ;
}


int TstObj::test_virtual() {

  return DB_VIRTUAL_RESULT ;
}


int TstObj::verify(int offs) {

  Ref<TstObj> myself ;
  yoo_getname(myself, this->_string) ;
  if (myself != this) return 0 ;

  if (this->_offs != offs) return 0 ;

  char string[512] ;
  sprintf(string, "Object number %d", offs + this->_first) ;
  if (strcmp(this->_string, string) != 0) return 0 ;

  if (this->_large[this->_largeSize - 1] != (char)(offs % 256)) return 0 ;

  if (this->_list.count() != offs) return 0 ;
  for (int i = 0; i < offs; i++) {
    if (this->_list[i] != i) return 0 ;
  }

  TstObjRoot* tmp = this ;
  if (tmp->test_virtual() != DB_VIRTUAL_RESULT) return 0 ;
  
  return 1 ;
}
  
void launchServer(char* s[]) {
  char command[512] ;

  for (int i = 0; i < DB_MAX_SERVER; i++) {
    sprintf(command, "yoo_server -b %s %s", DB_NAME, s[i]) ;
    system(command) ;
  }
}


int connect(char* name) {
  
  int res = yoo_connect(DB_NAME, name) ;
  yoo_getname(Main, "Main") ;
  if (Main == NULL) {
    Main = new MainObj ;
    Main->size = 0 ;
    Main->clusters += YOO_DEF_CLU ;
    Main->clusters += YoodaDictionary->getCluster("cluster1") ;
    Main->clusters += YoodaDictionary->getCluster("cluster2") ;
    Main->clusters += YoodaDictionary->getCluster("cluster3") ;
    yoo_setname(Main, "Main") ;
  }
  return res ;
}


int disconnect() {
  
  return yoo_disconnect() ;
}


Ref< ListX<TstObj> > create(int first, int num, ListX<Cluster>& l_clu) {
  int numclu = l_clu.count() ;
  Ref< ListX<TstObj> > l_obj = new ListX<TstObj> ;
  
  for (int i = 0; i < num; i++) {
    int clu = random() % numclu ;
    (*l_obj) += new (l_clu[clu]) TstObj(i, first) ;
  }

  return l_obj ;
}


int modify(const Ref< ListX<TstObj> >& l_obj) {

  int last ;
  last = yoo_alignSup(l_obj->count(), 2) / 2 ;
  for (int i = 0; i < last; i++) {
    if ((*l_obj)[i]->modify((*l_obj)[l_obj->count()-i-1]) == 0) {
      return 0 ;
    }
  }

  return 1 ;
}


int destroy(const Ref< ListX<TstObj> >& l_obj) {

  for (int i = 0; i < l_obj->count(); i++) {
    delete (TstObj*) (*l_obj)[i] ;
  }

  return 1 ;
}


int verify(const Ref< ListX<TstObj> >& l_obj) {

  for (int i = 0; i < l_obj->count(); i++) {
    if ((*l_obj)[i]->verify(i) == 0) {
      return 0 ;
    }
  }
  return 1 ;
}


int verify_destroy(const Ref< ListX<TstObj> >& l_obj) {

  for (int i = 0; i < l_obj->count(); i++) {
    if ((*l_obj)[i] != NULL) {
      return 0 ;
    }
  }
  return 1 ;
}


void yoo_alarm(int value)
{
  struct itimerval timeval ;

  timeval.it_value.tv_usec = value ;
  timeval.it_value.tv_sec = 0 ;
  timeval.it_interval.tv_usec = 0 ;
  timeval.it_interval.tv_sec = 0 ; 
  setitimer(ITIMER_REAL, &timeval, NULL) ;
}


int interrupt_commit() {
  yoo_alarm(DB_ALARM_DELAY) ;
  yoo_commit() ;
  return 1 ;
}


int interrupt_abort() {
  yoo_alarm(DB_ALARM_DELAY) ;
  yoo_abort() ;
  return 1 ;
}


